/************************************************************************
*  Modification History - In reverse time order.
*
*   January87	(mods by Larry Phillips)
*		- Updated documentation file to reflect Phone Book
*		- Correected bug in AtermClock that cause Jan/Feb
*		  to display month as "???"
*
*   December86	(mods by Jeff Lydiatt)
*		-Fixed "first file of wildcard transfer" bug
*		 Will now take directory name instead of using CD.
*
*   December86  (mods by Larry Phillips)
*		-merged Jeff's Phone Book version with CHAT version
*		-fixed FKey file bug... baud rate was remaining set to
*		 Preferences rather than being taken from FKey file.
*		-Changed CHAT menu item to toggle, checked.
*		-Added command sequence keys to CHAT and Phone Book.
*		-Added "Load FKeys" to Phone book.
*		-AtermClock may now be loaded from CD or C:
*
*   December86	(mods by Jeff Lydiatt)
*		-Added Phone Book.
*
*   November86	(mods by Larry Phillips)
*		-added CHAT mode. Split screen. Made it compatible with
*		 both 64 and 80 column text. Needs more generalized
*		 text width/height handling.
*
*   October86	(mods by Larry Phillips)
*		-added ability to load another function key file from
*		 a function key definition. <LOAD>
*		-added the ability to chain to an existing function
*		 key definition from any function key definition. <Fn:>
*		-added ability to load a function key file and chain to
*		 any of the newly defined function keys. <LOADEX>
*		 NOTE: Any parameters normally set by a function key file
*		 will be set by the newly loaded file. Things that are
*		 safe to change are Xon/off, protocol, autochop, prompt,
*		 and full/half/echo.
*
*   September86 -(mods by Jeff Lydiatt)
*		 Added SuperKermit routines.
*		 Fixed small bug in Xmodem CRC routine.  Bad blocks wern't
*		 being detected by the CRC algorithm.
*
*   August86	(mods by Jeff Lydiatt)
*		-Xmodem Chop won't chop 1A00 from downloaded files.  Arc 
*		 files seem to all end in this combination and complain
*		 if the last '00'x is chopped.  This fixes an annoying,
*		 but non-fatal warning message when you unarc the file.
*		-The standard pad character is now '00'x instead of CPM_EOF,
*		 in sympathy for those terminal programs not having a smart
*		 Chop routine.
*
*		(mods by Larry Phillips)
*   August 86	- Added <right Amiga> W for function key file SAVE (Menu.c)
*		- Search for function key file in S: directory if not
*		  found in current directory (aterm.c)
*		- Search for Clock (AtermClock) in C: directory if not found
*		  in current directory. (clockloader.c)
*
* (following major mods by Jeff Lydiatt, Vancouver, Canada)
*   August 86	- Added AtermClock, a multiprocess clock/calendar. 
*		- Added hunk_end padding for executable file uploads.
*   July 86	- Major reorganization of Aterm as follows:
*		- Program split into several Modules: Aterm.c,
*		  ConsoleIO.c, SerialIO.c, FKeys.c, Xmodem.c, Timer.c and
*		  Menu.c to make it easier to add new features.
*		- Aterm.c is the main program. ConsoleIO.c, and SerialIO.c
*		  are stand alone programs which manage the Console and
*		  the serial bus.  Menu.c is the output from the Mantis
*		  Menu maker, extensively modified by hand to bypass several
*		  pages of compiler error messages, add command keys, and
*		  include some necessary Menu utility procedures.
*		  Xmodem, and FKeys handle the Xmodem protocol
*		  and the function keys respectively.  Timer.c contains all
*		  the code to open, close and service one timer interrupt.
*		- Note that I have used the static keyword to hide variables
* 		  and functions that are only used inside the module.  There
*		  is now only one External variable, "KeyBuf", the
*		  function key buffer.
*		- Also note that I have gone to some lengths to make the
*		  init.key file upwardly compatible.  Your old keyfile 
*		  will not work with this version, but any future versions
*		  should be able to read the keyfile written by this one. 
*		- Also made extensive mods to make the code more readable
*		  tighter and (hopefully) more bug free.
*		- I also tried to keep the code relatively compiler indep-
*		  endant.  This version if still dependent on the Aztec
*		  compiler principally because Lattice has no equivalent
*		  for Aztec's "format" procedure.  If you have Lattice
*		  (Who knows?  Maybe the new version of Lattice might be
*		  worth using.) and wish to make mods to this code, you
*		  might try to write a format program using the standard
*		  sprintf() function!
*		- Added XmodemCRC cyclic redundency/ checksum XModem code,
*		  a highly modified version of Djj's Aterm1.3 xmodem
*		  code. This one includes auto chop; file size statistics 
*		  and time-to-send estimate; allows you to break in anywhere
*		  via the escape key; and uses the timer to check for
*		  timeouts.  (Aterm6.1 constantly polls the serial port and
*		  detects a timeout by checking the elapsed seconds using
*		  the AmigaDos DateStamp() function.)
*		- Copyright?  This is a Public Domain Program, including
*		  my mods!  Use it freely to add improvements or make new
*		  (hopefully public domain) programs.  However, may ill
*		  fortune fall upon you; you lose your Amiga to your worst
*		  enemy in a poker game; be forced to move into a one bedroom
*		  house, and your mother-in-law decide to live with you
*		  forever; if you have the cheek to try to sell this code...
*
*		(following mods by Larry Phillips)
*   June 86	- Disabled printing of CTRL N.
*		- Fixed hang when cursor key pressed.
*		  NOTE: RKM states that cursor keys are <CSI> [A...D] ~
*		  However, the ~ (tilde) is not part of the cursor key
*		  sequence. Proper sequence is <CSI> [A...D]
*
*		( these mods by Steve Allen)
*   June 86	- added xon/xoff control (default is OFF).
*		- expanded function-key file capabilities, adding info
*		  about prompt, baudrate, autochop, mode, xon/xoff.
*		- added code suggested by Larry Phillips/ICUG, to buffer
*		  serial input going to screen.
*		- adjusted Ascii capture to accept <TAB>.
*   May 86	- made all local screen writes red.
*   April 86	- switched order of CheckKey() and CheckIO() in
*		  SerIOGet(), to speed up Xmodem download.
*		- added AutoChop disable.
*		- added necessary code for PutFormat function in con.h
*		  buffered "printf()" (makes for speedy screen writes).
*   March 86	- added function-keys.
*		- changed keyboard and screen io to console.device,
*		  which allows ansi standard vt-100 emulation.
*   Feb 1986	- added Half-duplex and Echoplex.
*		- added timing to and fixed bugs in XMODEM receive.
*		- changed Ascii Send to Prompted Upload.
*		- added Command-Key sequences to menu.
*
************************************************************************/
